from base64 import urlsafe_b64decode
from django.conf import settings
import json
import jwt
import requests
from urllib import urlencode


DOMAIN = getattr(settings, 'AUTH0_DOMAIN')
SCOPE = getattr(settings, 'AUTH0_SCOPE', 'openid email')
CLIENT_ID = getattr(settings, 'AUTH0_CLIENT_ID')
CLIENT_SECRET = getattr(settings, 'AUTH0_CLIENT_SECRET')


def get_login_url(domain=DOMAIN, scope=SCOPE, client_id=CLIENT_ID, redirect_uri=None, response_mode='form_post', state=None):
    param_dict = {
        'response_type': 'token',
        'response_mode': response_mode,
        'scope': scope,
        'client_id': client_id,
    }
    if redirect_uri is not None:
        param_dict['redirect_uri'] = redirect_uri
    if state is not None:
        param_dict['state'] = state
    params = urlencode(param_dict)
    return 'https://{domain}/authorize?{params}'.format(
        domain=domain,
        params=params,
    )


def get_logout_url(redirect_uri, client_id=CLIENT_ID, domain=DOMAIN):
    params = urlencode({
        'returnTo': redirect_uri,
        'client_id': client_id,
    })
    return 'https://{domain}/v2/logout?{params}'.format(
        domain=domain,
        params=params,
    )


def get_email_from_token(token=None, key=urlsafe_b64decode(CLIENT_SECRET), audience=CLIENT_ID):
    try:
        payload = jwt.decode(token, key=key, audience=audience, leeway=300)
        if 'email' in payload:
            return payload['email']
        elif 'sub' in payload:
            return payload['sub'].split('|').pop()
    except (jwt.InvalidTokenError, IndexError) as e:
        pass

    return None


def is_email_verified_from_token(token=None, key=urlsafe_b64decode(CLIENT_SECRET), audience=CLIENT_ID):
    try:
        payload = jwt.decode(token, key=key, audience=audience, leeway=300)
        return payload.get('email_verified', False)
    except (jwt.InvalidTokenError, IndexError) as e:
        pass

    return None



from .utils import get_email_from_token, is_email_verified_from_token, get_login_url, get_logout_url
from base64 import urlsafe_b64encode
from django.conf import settings
try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User

    def get_user_model(*args, **kwargs):
        return User
from hashlib import sha1


class Auth0Backend(object):
    USER_CREATION = getattr(settings, 'AUTH0_USER_CREATION', True)

    supports_anonymous_user = False
    supports_inactive_user = True
    supports_object_permissions = False

    def __init__(self):
        self.User = get_user_model()

    def login_url(self, redirect_uri, state):
        return get_login_url(
            redirect_uri=redirect_uri,
            state=state
        )

    def logout_url(self, redirect_uri):
        return get_logout_url(redirect_uri=redirect_uri)

    def authenticate(self, token=None, **kwargs):
        if token is None:
            return None

        email = get_email_from_token(token=token)

        if email is None:
            return None

        if not is_email_verified_from_token(token=token):
            return None

        users = self.User.objects.filter(email=email)
        if len(users) == 0:
            user = self.create_user(email)
            if user is None:
                return None
        elif len(users) == 1:
            user = users[0]
        else:
            return None
        user.backend = '{}.{}'.format(self.__class__.__module__, self.__class__.__name__)
        return user

    def get_user(self, user_id):
        try:
            user = self.User.objects.get(pk=user_id)
            return user
        except self.User.DoesNotExist:
            return None

    def create_user(self, email):
        if self.USER_CREATION:
            username_field = getattr(self.User, 'USERNAME_FIELD', 'username')
            user_kwargs = {'email': email}
            user_kwargs[username_field] = self.username_generator(email)
            return self.User.objects.create_user(**user_kwargs)
        else:
            return None

    @staticmethod
    def username_generator(email):
        return urlsafe_b64encode(sha1(email).digest()).rstrip(b'=')



from .backends import Auth0Backend
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME, login
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, resolve_url
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.cache import never_cache
from urlparse import urlparse
import uuid


@never_cache
def auth(request):
    backend = Auth0Backend()
    redirect_uri = request.build_absolute_uri(reverse(callback))
    state = str(uuid.uuid4())
    request.session['state'] = state
    login_url = backend.login_url(
        redirect_uri=redirect_uri,
        state=state,
    )
    return HttpResponseRedirect(login_url)


@never_cache
def logout(request):
    backend = Auth0Backend()
    redirect_uri = request.build_absolute_uri(resolve_url(settings.LOGOUT_URL))
    logout_url = backend.logout_url(
        redirect_uri=redirect_uri,
    )
    return HttpResponseRedirect(logout_url)


@never_cache
@csrf_exempt
def callback(request):
    backend = Auth0Backend()
    original_state = request.session.get('state')
    state = request.POST.get('state')
    if original_state == state:
        token = request.POST.get('id_token')
        user = backend.authenticate(token=token)
        if user is not None:
            login(request, user)
            return HttpResponseRedirect(get_login_success_url(request))
    return redirect('auth0_login')


def get_login_success_url(request):
    redirect_to = request.GET.get(REDIRECT_FIELD_NAME, '')
    netloc = urlparse(redirect_to)[1]
    if not redirect_to:
        redirect_to = settings.LOGIN_REDIRECT_URL
    elif netloc and netloc != request.get_host():
        redirect_to = settings.LOGIN_REDIRECT_URL
    return redirect_to



from django.conf import settings
from django.core.urlresolvers import reverse
import uuid


def auth0(request):
    state = request.session.get('state', str(uuid.uuid4()))
    request.session['state'] = state

    return {
        'AUTH0_DOMAIN': getattr(settings, 'AUTH0_DOMAIN'),
        'AUTH0_CLIENT_ID': getattr(settings, 'AUTH0_CLIENT_ID'),
        'AUTH0_CLIENT_SECRET': getattr(settings, 'AUTH0_CLIENT_SECRET'),
        'AUTH0_SCOPE': getattr(settings, 'AUTH0_SCOPE', 'openid email'),
        'AUTH0_CALLBACK_URL': request.build_absolute_uri(reverse('auth0_callback')),
        'AUTH0_STATE': state,
    }



from .views import auth, callback, logout
from django.conf.urls import url


urlpatterns = [
    url(r'^login/$', auth, name='auth0_login'),
    url(r'^logout/$', logout, name='auth0_logout'),
    url(r'^callback/$', callback, name='auth0_callback'),
]



import os
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


def get_readme():
    """Return the README file contents. Supports text,rst, and markdown"""
    for name in ('README', 'README.rst', 'README.md'):
        if os.path.exists(name):
            return read_file(name)
    return ''

setup(
    name = 'django-auth0-auth',
    version = __import__('auth0_auth').get_version().replace(' ', '-'),
    url = 'https://bitbucket.org/megaman821/django-auth0-auth',
    author = 'Jason Christa',
    author_email = 'jason@zeitcode.com',
    description = 'Authenticated users using Auth0.',
    long_description = get_readme(),
    packages = find_packages(exclude=['tests']),
    include_package_data = True,
    install_requires = read_file('requirements.txt'),
    classifiers = [
        'Environment :: Web Environment',
        'License :: OSI Approved :: BSD License',
        'Framework :: Django',
        'Programming Language :: Python',
    ],
)
